<div
  style="display: grid;
         grid-template-rows: 10% 90%;
         grid-template-columns: 1fr 1fr;
         height: 360px;
         width: 100%;
         box-sizing: border-box;
         padding: 12px;
         background-color: #E8E1D4;
         border-radius: 6px;
         color: #413F3B;
        "
>
  <div style="grid-column: 1; grid-row: 1;">
  <span style="font-weight: 600;">Scheduler Status</span>
  </div>

  <div style="grid-column: 2; grid-row: 1;">
  <span style="font-weight: 600;">Worker Status</span>
  </div>

  <div style="grid-column: 1; grid-row: 2; border: solid; overflow: auto; margin-right: 6px">
    <pre style="padding: 12px; box-sizing: border-box; white-space: pre; min-height: 100%; background-color: white; color: black;">
      {%- for state in scheduler_status_logs | reverse -%}
        <code>--> {{ state | format_state }}</code><br>
      {%- endfor -%}
    </pre>
  </div>

  <div style="grid-column: 2; grid-row: 2; border: solid; overflow: auto;">
    <pre style="padding: 12px; box-sizing: border-box; white-space: pre; min-height: 100%; background-color: white; color: black;">
      {%- for state in worker_status_logs | reverse -%}
        <code>--> {{ state | format_state }}</code><br>
      {%- endfor -%}
    </pre>
  </div>
</div>
