<div
  style="
    color: #413F3B;
    width: auto;
    height: 360px;
    box-sizing: border-box;
    padding: 12px;
    background-color: #E8E1D4;
    border-radius: 6px;
    display: flex;
    flex-direction: column;
    justify-content: center;
    line-height: 3em;
  "
>
  <div><span style="font-weight: 600;">Cloud:&nbsp;</span>{% if cloud_name is not none %}{{ cloud_name }}{% else %}Not available{% endif %}</div>
  <div><span style="font-weight: 600;">Region:&nbsp;</span>{% if region_name is not none %}<code>{{ region_name }}</code>{% else %}Not available{% endif %}</div>
  <div><span style="font-weight: 600;">Scheduler Instance Type:&nbsp;</span>{% if scheduler_instance_type is not none %}<code>{{ scheduler_instance_type }}</code>{% else %}Not available{% endif %}</div>
  <div><span style="font-weight: 600;">Worker Instance Type(s):&nbsp;</span>{% if worker_instance_type is not none %}<code>{{ worker_instance_type }}</code>{% else %}Not Available{% endif %}</div>
  <div><span style="font-weight: 600;">Software Environment:&nbsp;</span>{% if software_environment_name is not none %}<code>{{ software_environment_name }}</code>{% else %}Not Available{% endif %}</div>
</div>
